<?php

namespace Btravel\Search\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Btravel\Search\Http\Requests\SearchRequest;
use Btravel\Search\Interfaces\SearchRepositoryInterface;
use Btravel\Search\Models\Search;

/**
 * Resource controller class for search.
 */
class SearchResourceController extends BaseController
{

    /**
     * Initialize search resource controller.
     *
     * @param type SearchRepositoryInterface $search
     *
     * @return null
     */
    public function __construct(SearchRepositoryInterface $search)
    {
        parent::__construct();
        $this->repository = $search;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Btravel\Search\Repositories\Criteria\SearchResourceCriteria::class);
    }

    /**
     * Display a list of search.
     *
     * @return Response
     */
    public function index(SearchRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Btravel\Search\Repositories\Presenter\SearchPresenter::class)
                ->$function();
        }

        $searches = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('search::search.names'))
            ->view('search::search.index', true)
            ->data(compact('searches', 'view'))
            ->output();
    }

    /**
     * Display search.
     *
     * @param Request $request
     * @param Model   $search
     *
     * @return Response
     */
    public function show(SearchRequest $request, Search $search)
    {

        if ($search->exists) {
            $view = 'search::search.show';
        } else {
            $view = 'search::search.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('search::search.name'))
            ->data(compact('search'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new search.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SearchRequest $request)
    {

        $search = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('search::search.name')) 
            ->view('search::search.create', true) 
            ->data(compact('search'))
            ->output();
    }

    /**
     * Create new search.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SearchRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $search                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('search::search.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('search/search/' . $search->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/search/search'))
                ->redirect();
        }

    }

    /**
     * Show search for editing.
     *
     * @param Request $request
     * @param Model   $search
     *
     * @return Response
     */
    public function edit(SearchRequest $request, Search $search)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('search::search.name'))
            ->view('search::search.edit', true)
            ->data(compact('search'))
            ->output();
    }

    /**
     * Update the search.
     *
     * @param Request $request
     * @param Model   $search
     *
     * @return Response
     */
    public function update(SearchRequest $request, Search $search)
    {
        try {
            $attributes = $request->all();

            $search->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('search::search.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('search/search/' . $search->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('search/search/' . $search->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the search.
     *
     * @param Model   $search
     *
     * @return Response
     */
    public function destroy(SearchRequest $request, Search $search)
    {
        try {

            $search->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('search::search.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('search/search/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('search/search/' . $search->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple search.
     *
     * @param Model   $search
     *
     * @return Response
     */
    public function delete(SearchRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('search::search.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('search/search'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/search/search'))
                ->redirect();
        }

    }

    /**
     * Restore deleted searches.
     *
     * @param Model   $search
     *
     * @return Response
     */
    public function restore(SearchRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('search::search.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/search/search'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/search/search/'))
                ->redirect();
        }

    }

}
